#include "sat/bsat/satVec.h"
#include "src/ccadical.h"

#define l_Undef 0
#define l_True 1
#define l_False -1
#define sat_solver CCaDiCaL

inline int abc2cadical (int l) {
  assert (l > 1);
  return (l >> 1) * (l & 1 ? -1 : 1);
}

inline int cadical2abc (int l) {
  assert (l);
  return (abs (l) << 1) + (l < 0);
}

inline sat_solver *glue_zsat_solver_new_seed (double seed) {
  return ccadical_init ();
}

inline sat_solver *glue_zsat_solver_restart_seed (sat_solver *s, double seed) {
  ccadical_release (s);
  return ccadical_init ();
}

inline int glue_sat_solver_nvars (sat_solver *s) { return ccadical_vars (s); }

inline void glue_sat_solver_setnvars (sat_solver *s, int n) {
  ccadical_reserve (s, n);
}

inline int glue_sat_solver_addclause (sat_solver *s, lit *begin, lit *end) {
  lit *i, l;
  assert (begin < end);
  for (i = begin; i < end; i++) {
    l = abc2cadical (*i);
    ccadical_add (s, l);
  }
  ccadical_add (s, 0);
  return true;
}

static inline int glue_sat_solver_final (sat_solver *s, int **ppArray) {
  int n = ccadical_get_core_assumptions (s, ppArray);
  for (int i = 0; i < n; ++i) {
    // abc expects the literals of a blocking clause ⇒ negation
    (*ppArray)[i] = cadical2abc (-(*ppArray)[i]);
    /* printf("failed %d\n", ((*ppArray)[i])); */
  }
  return n;
}

inline int glue_sat_solver_set_activation (sat_solver *s, lit activation){
  lit l = abc2cadical(activation);
  ccadical_set_activation (s, l);
}

inline int glue_sat_solver_solve (sat_solver *s, lit *begin, lit *end,
                                  ABC_INT64_T nConfLimit, ABC_INT64_T nInsLimit,
                                  ABC_INT64_T nConfLimitGlobal,
                                  ABC_INT64_T nInsLimitGlobal) {
  ccadical_clear_assumptions (s);
  lit *i, l;
  assert (begin <= end);
  for (i = begin; i < end; i++) {
    l = abc2cadical (*i);
    ccadical_assume (s, l);
  }
  int result = ccadical_solve (s);
  if (result == 10) {
    // prevent using core_assumptions after SAT call
    ccadical_clear_assumptions (s);
    return l_True;
  } else if (result == 20) {
    // save core assupmtions (blocking clause)
    ccadical_update_core_assumptions (s);
    return l_False;
  }
  return l_Undef;
}

static inline int glue_sat_solver_var_value (sat_solver *s, int v) {
  // Abc_vars map directly to positive literals
  assert (v > 0);
  return ccadical_val (s, v) > 0;
}

static inline int timesup (void *pLimit) {
  abctime limit = *((abctime *)pLimit);
  return limit && (limit > Abc_Clock ());
}

static inline abctime glue_sat_solver_set_runtime_limit (sat_solver *s,
                                                         abctime Limit) {
  abctime *pLimit = ccadical_get_runtime_limit (s);
  abctime previousLimit = *pLimit;
  *pLimit = Limit;
  ccadical_set_terminate (s, pLimit, timesup);
  return previousLimit;
}

static inline void glue_sat_solver_set_stop_func (sat_solver *s,
                                                  int (*fnct) (int)) {
  ccadical_set_terminate (s, NULL, (int (*) (void *))fnct);
}

static inline void glue_sat_solver_compress (sat_solver *s) {
  /* is way to expensive */
  /* ccadical_simplify(s); */
}

// adds a clause that has the same lifetime as an assumption
inline int sat_solver_constrain (sat_solver *s, lit *begin, lit *end) {
  lit *i, l;
  assert (begin < end);
  for (i = begin; i < end; i++) {
    l = abc2cadical (*i);
    ccadical_constrain (s, l);
  }
  ccadical_constrain (s, 0);
  return true;
}
