#!/bin/sh
die  () {
  echo "submit.sh: error: $*" 1>&2
  exit 1
}
[ -f config.sh ] || die "could not read 'config.sh'"
. ./config.sh
[ -x $binary ] || die "path '$binary' is not an executable"
version="`$binary --version 2>/dev/null|awk '{print $1}'`"
[ x"$version" = x ] && die "could not get version"
if [ $# -gt 0 ]
then
  [ x"$options" = x ] || options="$options "
  options="${options}$*"
  if [ ! -d $version ]; then mkdir $version || exit 1; fi
  subdir="`echo -- $*|fmt -0|fmt -1000|sed -e 's,[ =],,g' -e 's,--*,-,g' -e 's,^-,,'`"
  dir=$version/"$subdir"
else
  dir=$version
fi
[ -d $dir ] && die "directory '$dir' already exists"
mkdir $dir || exit 1
benchmarks="`wc -l benchmarks|awk '{print $1}'`"
[ x"$benchmarks" = x ] && die "could not get number of benchmarks"
echo "run        $run"
echo "cpus       $cpus"
echo "time       $time"
echo "space      $space"
echo "binary     $binary"
echo "options    $options"
echo "version    $version"
echo "benchmarks $benchmarks"
echo "directory  $dir"
install $binary $dir/binary || exit 1
sed -e "s,@RUN@,$run," \
    -e "s,@CPUS@,$cpus," \
    -e "s,@BENCHMARKS@,$benchmarks," array.sh > $dir/array.sh
sed -e "s,^options=.*,options=\"$options\"," config.sh > $dir/config.sh
chmod 755 $dir/array.sh $dir/config.sh
cp -p benchmarks $dir
cd $dir
exec sbatch ./array.sh
